@@ -29,9 +29,14 @@
 #include <QDirIterator>
 #include <QFileDialog>
 #include <QHash>
+#include <QLineEdit>
+#include <QLabel>
+#include <QMessageBox>
 
 #include "xfwm_compositor_settings.h"
 #include "window_buttons.h"
+#include "theming_to_tweak.h"
+#include "remove_user_theme_set.h"
 
 defaultlook::defaultlook(QWidget *parent) :
     QDialog(parent),
@@ -1153,7 +1158,7 @@ void defaultlook::on_buttonThemeApply_clicked()
     QString themename = theme_info[ui->comboTheme->currentText()];
     QFileInfo fileinfo(themename);
     //initialize variables
-    QString backgroundColor = runCmd("cat '" + fileinfo.absoluteFilePath() + "' |grep background-color=").output.section("=" , 1,1);
+    QString backgroundColor = runCmd("cat '" + fileinfo.absoluteFilePath() + "' |grep background-rgba=").output.section("=" , 1,1);
     qDebug() << "backgroundColor = " << backgroundColor;
     QString color1 = backgroundColor.section(",",0,0);
     QString color2 = backgroundColor.section(",", 1, 1);
@@ -1222,7 +1227,7 @@ void defaultlook::on_buttonThemeApply_clicked()
         //set panel color
 
         if (backgroundColor != "") {
-            runCmd("xfconf-query -c xfce4-panel -p /panels/panel-" + value + "/background-color -t uint -t uint -t uint -t uint -s " + color1 + " -s " + color2 + " -s " + color3 + " -s " + color4 + " --create");
+            runCmd("xfconf-query -c xfce4-panel -p /panels/panel-" + value + "/background-rgba -t double -t double -t double -t double -s " + color1 + " -s " + color2 + " -s " + color3 + " -s " + color4 + " --create");
         }
 
 
@@ -1790,4 +1795,134 @@ QString defaultlook::getVersion(QString name)
     return runCmd("dpkg-query -f '${Version}' -W " + name).output;
 }
 
+void defaultlook::on_pushButtonSettingsToThemeSet_clicked()
+{
+    QString fileName;
+    theming_to_tweak* dialog = new theming_to_tweak;
+    int userInput = dialog->exec();
+    if(userInput == QDialog::Rejected)
+        return;
+    fileName = dialog->nameEditor()->text();
 
+    //declared locally to prevent an issues with other code
+    auto pathAppend = [](const QString& path1, const QString& path2)
+    {
+        return QDir::cleanPath(path1 + QDir::separator() + path2);
+    };
+
+    QString panel;
+    QString data = runCmd("xfconf-query -c xfce4-panel -p /panels --list").output;
+    int panelNum;
+    for(panelNum = 1;; panelNum++)
+    {
+        if(data.contains("panel-" + QString::number(panelNum)))
+            break;
+    }
+    panel = "panel-" + QString::number(panelNum);
+
+    int backgroundStyle;
+    data = runCmd("xfconf-query -c xfce4-panel -p /panels/" + panel + "/background-style").output;
+    backgroundStyle = data.toInt(); //there may be newlines in output but qt ignores it
+
+    QVector<double> backgroundColor;
+    QString backgroundImage;
+    if(backgroundStyle == 1)
+    {
+        QStringList lines = runCmd("xfconf-query -c xfce4-panel -p /panels/" + panel + "/background-rgba").output.split('\n');
+        lines.removeAt(0);
+        lines.removeAt(0);
+        for(int i = 0; i < 4; i++)
+        {
+            backgroundColor << lines.at(i).toDouble();
+        }
+    }
+    else if(backgroundStyle == 2)
+    {
+        backgroundImage = runCmd("xfconf-query -c /panels/" + panel + "/background-image").output;
+    }
+
+    QString iconThemeName = runCmd("xfconf-query -c xsettings -p /Net/IconThemeName").output;
+    QString themeName = runCmd("xfconf-query -c xsettings -p /Net/ThemeName").output;
+    QString windowDecorationsTheme = runCmd("xfconf-query -c xfwm4 -p /general/theme").output;
+
+    QString whiskerThemeFileName = pathAppend(QDir::homePath(), ".config/gtk-3.0/whisker-tweak.css");
+    QFile whiskerThemeFile(whiskerThemeFileName);
+    if(!whiskerThemeFile.open(QFile::ReadOnly | QFile::Text))
+    {
+        qDebug() << "Failed to fetch whisker theming from: " + whiskerThemeFileName;
+    }
+    QTextStream whiskerThemeFileStream(&whiskerThemeFile);
+    QString whiskerThemeData = whiskerThemeFileStream.readAll();
+    whiskerThemeFile.close();
+
+    QStringList fileLines;
+    fileLines << "Name=" + fileName;
+    fileLines << "background-style=" + QString::number(backgroundStyle);
+    if(backgroundStyle == 1)
+    {
+        QString line;
+        for(double num : backgroundColor)
+        {
+            line.append(QString::number(num) + ',');
+        }
+        if(line.endsWith(',')) line.chop(1);
+        fileLines << "background-rgba=" + line;
+    }
+    else
+    {
+        fileLines << "background-rgba=none";
+    }
+    if(backgroundStyle == 2)
+    {
+        fileLines << "background-image=" + backgroundImage;
+    }
+    else
+    {
+        fileLines << "background-image=none";
+    }
+    fileLines << "xsettings_gtk_theme=" + themeName;
+    fileLines << "xsettings_icon_theme=" + iconThemeName;
+    fileLines << "xfwm4_window_decorations=" + windowDecorationsTheme;
+    fileLines << "<begin_gtk_whisker_theme_code>";
+    for(QString line : whiskerThemeData.split('\n'))
+    {
+        fileLines << line;
+    }
+    fileLines << "<end_gtk_whisker_theme_code>";
+    QFile file(pathAppend(QDir::homePath(), ".local/share/mx-tweak-data/" + fileName + ".tweak"));
+    if(!file.open(QFile::WriteOnly | QFile::Text))
+    {
+        qDebug() << "Failed to open file for reading: " + fileName;
+        return;
+    }
+    QTextStream fileStream(&file);
+    for(QString line : fileLines)
+    {
+        fileStream << line + '\n';
+    }
+    file.close();
+    //Refresh
+    setupComboTheme();
+}
+
+void defaultlook::on_pushButtonRemoveUserThemeSet_clicked()
+{
+    remove_user_theme_set dialog;
+    int result = dialog.exec();
+    if(result != QDialog::Accepted)
+        return;
+    QString theme = dialog.themeSelector()->currentText();
+    result = QMessageBox::warning(this, "Remove User Theme Set", "Are you sure you want to remove " + theme + " theme set?", QMessageBox::Ok | QMessageBox::Cancel);
+    if(result != QMessageBox::Ok)
+        return;
+    QString file = dialog.getFilename(theme);
+    file.replace(' ', "\\ ");
+    auto cmd = runCmd("rm " + file);
+    if(cmd.exitCode != 0)
+    {
+        qDebug() << "Removing theme set failed: exitCode: " << cmd.exitCode << " | output: " << cmd.output;
+        return;
+    }
+    //refresh
+    setupComboTheme();
+}
